// Definicija apstraktne klase za zbirke figura (Zbirka).

#ifndef _zbirka_h_
#define _zbirka_h_

#include <iostream>
using namespace std;

#include "figura1.h"                          // Definicija klase Figura.
class Iter;                                   // Deklaracija klase Iter.

class Zbirka {
  virtual void kopiraj(const Zbirka& z) {}    // Kopiranje u zbirku.
  virtual void premesti(Zbirka& z) {}         // Premestanje u zbirku.
  virtual void brisi() {}                     // Oslobadjanje memorije.
public:
  Zbirka() =default;                          // Podrazumevani konstruktor.
  Zbirka(const Zbirka& z) { kopiraj(z); }     // Kopirajuci konstruktor.
  Zbirka(Zbirka&& z) { premesti(z); }         // Premestajuci konstruktor.
  Zbirka& operator=(const Zbirka& z) {        // Kopirajuca dodela
    if (this != &z) { brisi(); kopiraj(z); }  //    vrednosti.
    return *this;
  }
  Zbirka& operator=(Zbirka&& z) {             // Premestajuca dodela
    if (this != &z) { brisi(); premesti(z); } //    vrednosti.
    return *this;
  }
  virtual ~Zbirka() { brisi(); }              // Virtuelan destruktor.
  virtual int vel() const =0;                 // Broj elemenata zbirke.
  virtual Zbirka& operator+=(Figura* fig)=0;  // Dodavanje figure.
  friend Zbirka& operator+=(Zbirka& zb, const Figura& fig)
    { return zb += fig.kopija(); }
  virtual Figura*& operator[](int ind) =0;    // Dohvatanje figure.
  const Figura* operator[](int ind) const
    { return const_cast<Zbirka&>(*this)[ind]; }
  virtual Zbirka& operator~() =0;             // Praznjenje zbirke.
  virtual Zbirka* kopija() const =0;          // Stvaranje kopije zbirke.
  friend ostream& operator<<                  // Pisanje zbirke.
    (ostream& it, const Zbirka& zb);

  virtual Iter* iter() =0;                    // Stvaranje iteratora.
  virtual const Iter* iter() const =0;

  enum { G_IND, G_TEK };                      // Sifre gresaka.
  static void greska(int g);                  // Prekid programa.
};

#endif

